/*
 *
 * Tachometer with HDSP-B03E display ver 1.0
 *
 * Juha Niinikoski 11.03.2005
 * 
 * Average filter added 13.03.2005
 * Changed to HDSP-B03E display & new PCB 30.03.2005
 * Unique calibration values for two first units 31.03.2005
 *
*/

#include <pic.h>
#include "revctr2.h"

/* global variables */

near bit s_flag;				// second elapsed
near unsigned char s0;			// second wait counters

static near unsigned char digit_data[4];	// segment bits
static near unsigned char dp_data[4];		// DP bits
static near unsigned char digit_sel;		// to maintain PORTA bits
static near unsigned char strobo;

bit run_flag;					// signal ok flag
bit mode;						// operating mode, 1 = waste spark
bit button;						// button pressed from comparator

bit meas_ready;					// measurement ready
unsigned int t1_result;			// timer1 result
unsigned int rpm;				// rpm / 10

#define NUM_AVG 4				// average size
unsigned int avg[NUM_AVG];		// average array
char idx_avg;					// array index
unsigned int rpm_avg;			// averaged rpm


// nominal values / calibration
// for accurate display 16F628 internal oscillator
// may require unique calibration

// Chip 02461GC
//#warning Cal 2 used
//#define two_stroke   7717500UL

// Chip 03362UM
//#warning Cal 3 used
//#define two_stroke   7747500UL
 
#define two_stroke   7500000UL	// constant for two stroke or waste spark 4 stroke engine, spark at every round
#define four_stroke (two_stroke * 2)	// constant for four stroke engine, two cranc rounds / spark

// Equation for two stroke (spark/round)
//
// 1 rpm = 60s, TMR1 clock = 8us
// 60s/8us = 7.500.000
// RPM Display = 7.500.000 / TMR1count
// 
// TMR1 max count = 65535, min RPM display = 114 RPM
//
// Four stroke (spark at every second roune) 
//
// Constant = 2 * 7.500.000
// Min RPM display = 228 RPM
//
// MAX RPM display limited by display to 9999


/* included programs */

#include "7segmux2.c"

/* processor config & id info */

	__CONFIG(UNPROTECT & INTIO & WDTDIS & PWRTEN & BOREN & LVPDIS & MCLRDIS);	// 16F628 & 16F81x

	__IDLOC(0103);							// Device ID

const char id[] = {"Juha Niinikoski 31.03.2005"};	// Visible id string

/* main  */

void main(void)
{
char x;

	CMCON = CMCON_INIT;			// comparators off 16F628 only
	VRCON = VRCON_INIT;			// voltage reference init

	PORTB = PORTB_INIT;
	PORTA = PORTA_INIT;

	TRISB = TRISB_INIT;			// RB5 - RB7 otputs
	TRISA = TRISA_INIT;

	init_timers();				// start timing functions

// display name
	digit_data[3] = C_J;		// display start up message
	digit_data[2] = C_U;	
	digit_data[1] = C_H;
	digit_data[0] = C_A;
	delayms(1500);

// display version
	digit_data[3] = C_NEG;	
	digit_data[2] = C_0;		// version number
	digit_data[1] = C_1;
	digit_data[0] = C_NEG;
	delayms(1500);

	for(;;)
		{

// if measurement ready calculate rpm
		if(meas_ready)
			{
			meas_ready = 0;				// reset ready flag
			msw = 1000;					// reset timeout

			run_flag = !run_flag;		// toggle display run flag

			if(mode)					// print result, spark / two rounds
				{
				rpm = four_stroke / t1_result;
				}
			else						// spark / round
				{
				rpm = two_stroke / t1_result;
				}

// average filter for result
			avg[idx_avg] = rpm;			// store last result
			idx_avg++;					// addjust index
			if(idx_avg >= NUM_AVG)
				idx_avg = 0;

			rpm_avg = 0;				// calculate average
			for(x=0; x< NUM_AVG; x++)
				{
				rpm_avg = rpm_avg + avg[x];
				}

			rpm_avg = rpm_avg + (NUM_AVG / 2);	// round
			rpm_avg = rpm_avg / NUM_AVG;

// display average
			putintdec(rpm_avg);

// no averaging
//			putintdec(rpm);

			}

// slow signal trap
		if(msw == 0)					// note msw is also used for ms delay
				{
				putst("-???");			// no signal display, save power
				run_flag = 0;			// blanc point, save power
				}

// set DP point flags for display
#if 0
		if (run_flag == 1)				// show run flag
			dp_data[2] = Dp;
		else
			dp_data[2] = 0;
#endif

		if (mode == 1)
			dp_data[1] = Dp;			// mode point (4 stroke select) lower dot
		else
			dp_data[1] = 0;

// evaluate mode change button
		if (button == 1)
			{
			mode = !mode;				// toggle mode
			delayms(400);				// debounce
			}

		}

}
